@@ -5,14 +5,14 @@
 	tape drive.
 
 	Maintained by Kai Mkisara (email Kai.Makisara@metla.fi)
-	Copyright by Kai Mkisara, 1998. The program may be distributed
+	Copyright by Kai Mkisara, 1998 - 2000. The program may be distributed
 	according to the GNU Public License
 
-	Last Modified: Sun Aug 16 09:51:17 1998 by makisara@home
+	Last Modified: Thu Nov 30 21:15:38 2000 by makisara@kai.makisara.local
 */
 
 #ifndef lint
-static char rcsid[] = "$Id: /usr2/users/makisara/src/sys/mt-st-0.5b/mt.c at Sun Aug 16 09:51:17 1998 by makisara@kai.makisara.fi$";
+static char rcsid[] = "$Id: /usr2/users/makisara/src/sys/mt-st-0.6/mt.c at Thu Nov 30 21:15:38 2000 by makisara@kai.makisara.local$";
 #endif
 
 #include <stdio.h>
@@ -20,16 +20,18 @@ static char rcsid[] = "$Id: /usr2/users/makisara/src/sys/mt-st-0.5b/mt.c at Sun
 #include <stdlib.h>
 #include <string.h>
 #include <ctype.h>
+#include <errno.h>
+#include <fcntl.h>
 #include <sys/types.h>
 #include <sys/ioctl.h>
-#include <sys/mtio.h>
-#include <fcntl.h>
 
-#define VERSION "0.5b"
+#include "mtio.h"
 
-/* Set to one to include the datcompression command (not necessary in
-   Linux 2.x) */
-#define DATCOMPRESSION 0
+#ifndef DEFTAPE
+#define DEFTAPE "/dev/tape"     /* default tape device */
+#endif /* DEFTAPE */
+
+#define VERSION "0.6"
 
 typedef int (* cmdfunc)(/* int, struct cmdef_tr *, int, char ** */);
 
@@ -40,6 +42,7 @@ typedef struct cmdef_tr {
     int cmd_count_bits;
     unsigned char cmd_fdtype;
     unsigned char arg_cnt;
+    int error_tests;
 } cmdef_tr;
 
 #define NO_FD      0
@@ -55,6 +58,9 @@ typedef struct cmdef_tr {
 #define SET_BOOLEANS   1003
 #define CLEAR_BOOLEANS 1004
 
+#define ET_ONLINE    1
+#define ET_WPROT     2
+
 static void usage(int);
 static int do_standard(int, cmdef_tr *, int, char **);
 static int do_drvbuffer(int, cmdef_tr *, int, char **);
@@ -64,80 +70,107 @@ static int do_partseek(int, cmdef_tr *, int, char **);
 static int do_status(int, cmdef_tr *, int, char **);
 static int print_densities(int, cmdef_tr *, int, char **);
 static int do_asf(int, cmdef_tr *, int, char **);
-#if DATCOMPRESSION
-static int do_dat_compression(int, cmdef_tr *, int, char **);
-#endif
-
+static void test_error(int, cmdef_tr *);
 
 static cmdef_tr cmds[] = {
-    { "weof",		MTWEOF,	  do_standard, 0, FD_RDWR,   ONE_ARG },
-    { "wset",		MTWSM,	  do_standard, 0, FD_RDWR,   ONE_ARG },
-    { "eof",		MTWEOF,	  do_standard, 0, FD_RDONLY, ONE_ARG },
-    { "fsf",		MTFSF,	  do_standard, 0, FD_RDONLY, ONE_ARG },
-    { "fsfm",		MTFSFM,	  do_standard, 0, FD_RDONLY, ONE_ARG },
-    { "bsf",		MTBSF,	  do_standard, 0, FD_RDONLY, ONE_ARG },
-    { "bsfm",		MTBSFM,	  do_standard, 0, FD_RDONLY, ONE_ARG },
-    { "fsr",		MTFSR,	  do_standard, 0, FD_RDONLY, ONE_ARG },
-    { "bsr",		MTBSR,	  do_standard, 0, FD_RDONLY, ONE_ARG },
-    { "fss",		MTFSS,	  do_standard, 0, FD_RDONLY, ONE_ARG },
-    { "bss",		MTBSS,	  do_standard, 0, FD_RDONLY, ONE_ARG },
-    { "rewind",		MTREW,	  do_standard, 0, FD_RDONLY, NO_ARGS },
-    { "offline",	MTOFFL,	  do_standard, 0, FD_RDONLY, NO_ARGS },
-    { "rewoffl",	MTOFFL,	  do_standard, 0, FD_RDONLY, NO_ARGS },
-    { "retension",	MTRETEN,  do_standard, 0, FD_RDONLY, NO_ARGS },
-    { "eod",		MTEOM,	  do_standard, 0, FD_RDONLY, NO_ARGS },
-    { "seod",		MTEOM,	  do_standard, 0, FD_RDONLY, NO_ARGS },
-    { "seek",		MTSEEK,   do_standard, 0, FD_RDONLY, ONE_ARG },
-    { "tell",		MTTELL,	  do_tell,     0, FD_RDONLY, NO_ARGS },
-    { "status",		MTNOP,	  do_status,   0, FD_RDONLY, NO_ARGS },
-    { "erase",		MTERASE,  do_standard, 0, FD_RDWR,   NO_ARGS },
-    { "setblk",		MTSETBLK, do_standard, 0, FD_RDONLY, ONE_ARG },
+    { "weof",		MTWEOF,	  do_standard, 0, FD_RDWR,   ONE_ARG,
+    ET_ONLINE | ET_WPROT },
+    { "wset",		MTWSM,	  do_standard, 0, FD_RDWR,   ONE_ARG,
+    ET_ONLINE | ET_WPROT },
+    { "eof",		MTWEOF,	  do_standard, 0, FD_RDWR, ONE_ARG,
+    ET_ONLINE },
+    { "fsf",		MTFSF,	  do_standard, 0, FD_RDONLY, ONE_ARG,
+    ET_ONLINE },
+    { "fsfm",		MTFSFM,	  do_standard, 0, FD_RDONLY, ONE_ARG,
+    ET_ONLINE },
+    { "bsf",		MTBSF,	  do_standard, 0, FD_RDONLY, ONE_ARG,
+    ET_ONLINE },
+    { "bsfm",		MTBSFM,	  do_standard, 0, FD_RDONLY, ONE_ARG,
+    ET_ONLINE },
+    { "fsr",		MTFSR,	  do_standard, 0, FD_RDONLY, ONE_ARG,
+    ET_ONLINE },
+    { "bsr",		MTBSR,	  do_standard, 0, FD_RDONLY, ONE_ARG,
+    ET_ONLINE },
+    { "fss",		MTFSS,	  do_standard, 0, FD_RDONLY, ONE_ARG,
+    ET_ONLINE },
+    { "bss",		MTBSS,	  do_standard, 0, FD_RDONLY, ONE_ARG,
+    ET_ONLINE },
+    { "rewind",		MTREW,	  do_standard, 0, FD_RDONLY, NO_ARGS,
+    ET_ONLINE },
+    { "offline",	MTOFFL,	  do_standard, 0, FD_RDONLY, NO_ARGS,
+    ET_ONLINE },
+    { "rewoffl",	MTOFFL,	  do_standard, 0, FD_RDONLY, NO_ARGS,
+    ET_ONLINE },
+    { "retension",	MTRETEN,  do_standard, 0, FD_RDONLY, NO_ARGS,
+    ET_ONLINE },
+    { "eod",		MTEOM,	  do_standard, 0, FD_RDONLY, NO_ARGS,
+    ET_ONLINE },
+    { "seod",		MTEOM,	  do_standard, 0, FD_RDONLY, NO_ARGS,
+    ET_ONLINE },
+    { "seek",		MTSEEK,   do_standard, 0, FD_RDONLY, ONE_ARG,
+    ET_ONLINE },
+    { "tell",		MTTELL,	  do_tell,     0, FD_RDONLY, NO_ARGS,
+    ET_ONLINE },
+    { "status",		MTNOP,	  do_status,   0, FD_RDONLY, NO_ARGS,
+    0 },
+    { "erase",		MTERASE,  do_standard, 0, FD_RDWR,   NO_ARGS,
+    ET_ONLINE },
+    { "setblk",		MTSETBLK, do_standard, 0, FD_RDONLY, ONE_ARG,
+    0 },
 #ifdef MTLOCK
-    { "lock",		MTLOCK,   do_standard, 0, FD_RDONLY, NO_ARGS },
-    { "unlock", 	MTUNLOCK, do_standard, 0, FD_RDONLY, NO_ARGS },
+    { "lock",		MTLOCK,   do_standard, 0, FD_RDONLY, NO_ARGS,
+    ET_ONLINE },
+    { "unlock", 	MTUNLOCK, do_standard, 0, FD_RDONLY, NO_ARGS,
+    ET_ONLINE },
 #endif
 #ifdef MTLOAD
-    { "load",		MTLOAD,	  do_standard, 0, FD_RDONLY, NO_ARGS },
+    { "load",		MTLOAD,	  do_standard, 0, FD_RDONLY, NO_ARGS,
+    0 },
 #endif
 #ifdef MTCOMPRESSION
-    { "compression",	MTCOMPRESSION,  do_standard,  0, FD_RDONLY, ONE_ARG},
+    { "compression",	MTCOMPRESSION,  do_standard,  0, FD_RDONLY, ONE_ARG,
+    0 },
 #endif
-    { "setdensity",	MTSETDENSITY,   do_standard,  0, FD_RDONLY, ONE_ARG},
-    { "drvbuffer",	MTSETDRVBUFFER, do_drvbuffer, 0, FD_RDONLY, ONE_ARG},
+    { "setdensity",	MTSETDENSITY,   do_standard,  0, FD_RDONLY, ONE_ARG,
+    0 },
+    { "drvbuffer",	MTSETDRVBUFFER, do_drvbuffer, 0, FD_RDONLY, ONE_ARG,
+    0 },
     { "stwrthreshold",	MTSETDRVBUFFER, do_drvbuffer, MT_ST_WRITE_THRESHOLD,
-      FD_RDONLY, ONE_ARG},
+      FD_RDONLY, ONE_ARG, 0},
     { "stoptions",	DO_BOOLEANS,    do_options,   0, FD_RDONLY,
-      MULTIPLE_ARGS},
+      MULTIPLE_ARGS, 0},
 #ifdef MT_ST_SETBOOLEANS
     { "stsetoptions",   SET_BOOLEANS,   do_options,   0, FD_RDONLY,
-      MULTIPLE_ARGS},
+      MULTIPLE_ARGS, 0},
     { "stclearoptions", CLEAR_BOOLEANS, do_options,   0, FD_RDONLY,
-      MULTIPLE_ARGS},
+      MULTIPLE_ARGS, 0},
     { "defblksize",	MTSETDRVBUFFER, do_drvbuffer, MT_ST_DEF_BLKSIZE,
-      FD_RDONLY, ONE_ARG},
+      FD_RDONLY, ONE_ARG, 0},
     { "defdensity",	MTSETDRVBUFFER, do_drvbuffer, MT_ST_DEF_DENSITY,
-      FD_RDONLY, ONE_ARG},
+      FD_RDONLY, ONE_ARG, 0},
     { "defdrvbuffer",	MTSETDRVBUFFER, do_drvbuffer, MT_ST_DEF_DRVBUFFER,
-      FD_RDONLY, ONE_ARG},
+      FD_RDONLY, ONE_ARG, 0},
     { "defcompression", MTSETDRVBUFFER, do_drvbuffer, MT_ST_DEF_COMPRESSION,
-      FD_RDONLY, ONE_ARG},
+      FD_RDONLY, ONE_ARG, 0},
 #endif
 #ifdef MT_ST_SET_TIMEOUT
     { "sttimeout",	MTSETDRVBUFFER, do_drvbuffer, MT_ST_SET_TIMEOUT,
-      FD_RDONLY, ONE_ARG},
+      FD_RDONLY, ONE_ARG, 0},
     { "stlongtimeout",	MTSETDRVBUFFER, do_drvbuffer, MT_ST_SET_LONG_TIMEOUT,
-      FD_RDONLY, ONE_ARG},
-#endif
-    { "densities",	0, print_densities,     0, NO_FD,     NO_ARGS},
-#if DATCOMPRESSION
-    { "datcompression", 0, do_dat_compression,  0, FD_RDWR,   ONE_ARG},
+      FD_RDONLY, ONE_ARG, 0},
 #endif
+    { "densities",	0, print_densities,     0, NO_FD,     NO_ARGS,
+    0 },
 #ifdef MTSETPART
-    { "setpartition",	MTSETPART, do_standard, 0, FD_RDONLY, ONE_ARG},
-    { "mkpartition",	MTMKPART,  do_standard, 0, FD_RDWR,   ONE_ARG},
-    { "partseek",	0,         do_partseek, 0, FD_RDONLY, TWO_ARGS},
+    { "setpartition",	MTSETPART, do_standard, 0, FD_RDONLY, ONE_ARG,
+    ET_ONLINE },
+    { "mkpartition",	MTMKPART,  do_standard, 0, FD_RDWR,   ONE_ARG,
+    ET_ONLINE },
+    { "partseek",	0,         do_partseek, 0, FD_RDONLY, TWO_ARGS,
+    ET_ONLINE },
 #endif
-    { "asf",		0,         do_asf, MTREW,  FD_RDONLY, ONE_ARG},
+    { "asf",		0,         do_asf, MTREW,  FD_RDONLY, ONE_ARG,
+    ET_ONLINE },
     { NULL, 0, 0, 0 }
 };
 
@@ -155,7 +188,7 @@ static struct densities {
     {0x06, "PE (3200 bpi)"},
     {0x07, "IMFM (6400 bpi)"},
     {0x08, "GCR (8000 bpi)"},
-    {0x09, "GCR /37871 bpi)"},
+    {0x09, "GCR (37871 bpi)"},
     {0x0a, "MFM (6667 bpi)"},
     {0x0b, "PE (1600 bpi)"},
     {0x0c, "GCR (12960 bpi)"},
@@ -169,14 +202,44 @@ static struct densities {
     {0x15, "EXB-8500 or QIC-1000"},
     {0x16, "MFM 10000 bpi"},
     {0x17, "MFM 42500 bpi"},
+    {0x18, "TZ86"},
+    {0x19, "DLT 10GB"},
+    {0x1a, "DLT 20GB"},
+    {0x1b, "DLT 35GB"},
+    {0x1c, "QIC-385M"},
+    {0x1d, "QIC-410M"},
+    {0x1e, "QIC-1000C"},
+    {0x1f, "QIC-2100C"},
+    {0x20, "QIC-6GB"},
+    {0x21, "QIC-20GB"},
     {0x22, "QIC-2GB"},
+    {0x23, "QIC-875"},
     {0x24, "DDS-2"},
     {0x25, "DDS-3"},
+    {0x26, "DDS-4 or QIC-4GB"},
+    {0x27, "Exabyte Mammoth"},
+    {0x28, "Exabyte Mammoth-2"},
+    {0x29, "QIC-3080MC"},
+    {0x30, "AIT-1 or MLR3"},
+    {0x31, "AIT-2"},
+    {0x33, "SLR6"},
+    {0x34, "SLR100"},
+    {0x41, "DLT 40GB"},
     {0x45, "QIC-3095-MC (TR-4)"},
-    {140, "EXB-8505 compressed"},
-    {144, "EXB-8205 compressed"},
-    {-1, NULL}};
+    {0x47, "TR-5"},
+    {0x80, "DLT 15GB uncompressed"},
+    {0x81, "DLT 15GB compressed"},
+    {0x82, "DLT 20GB uncompressed"},
+    {0x83, "DLT 20GB compressed"},
+    {0x84, "DLT 35GB uncompressed"},
+    {0x85, "DLT 35GB compressed"},
+    {0x88, "DLT 40GB uncompressed"},
+    {0x89, "DLT 40GB compressed"},
+    {0x8c, "EXB-8505 compressed"},
+    {0x90, "EXB-8205 compressed"}
+};
 
+#define NBR_DENSITIES (sizeof(density_tbl) / sizeof(struct densities))
 
 static struct booleans {
     char *name;
@@ -206,7 +269,7 @@ static char *tape_name;   /* The tape name for messages */
 	int
 main(int argc, char **argv)
 {
-    int mtfd, cmd_code, i, argn, len;
+    int mtfd, cmd_code, i, argn, len, oflags;
     char *cmdstr;
     cmdef_tr *comp, *comp2;
 
@@ -230,6 +293,13 @@ main(int argc, char **argv)
 		printf("mt-st v. %s\n", VERSION);
 		exit(0);
 		break;
+	    case '-':
+		if (*(argv[argn] + 1) == '-' &&
+		    *(argv[argn] + 2) == 'v') {
+		    printf("mt-st v. %s\n", VERSION);
+		    exit(0);
+		}
+		/* Fall through */
 	    default:
 		usage(0);
 		exit(1);
@@ -273,8 +343,10 @@ main(int argc, char **argv)
     cmd_code = comp->cmd_code;
 
     if (comp->cmd_fdtype != NO_FD) {
-	if ((mtfd = open(tape_name,
-			 comp->cmd_fdtype == FD_RDONLY ? O_RDONLY : O_RDWR)) < 0) {
+	oflags = comp->cmd_fdtype == FD_RDONLY ? O_RDONLY : O_RDWR;
+	if ((comp->error_tests & ET_ONLINE) == 0)
+	    oflags |= O_NONBLOCK;
+	if ((mtfd = open(tape_name, oflags)) < 0) {
 	    perror(tape_name);
 	    exit(1);
 	}
@@ -282,9 +354,16 @@ main(int argc, char **argv)
     else
 	mtfd = (-1);
 
-    if (comp->cmd_function != NULL)
+    if (comp->cmd_function != NULL) {
 	i = comp->cmd_function(mtfd, comp, argc - argn,
 			       (argc - argn > 0 ? argv + argn : NULL));
+	if (i) {
+	    if (errno == ENOSYS)
+		fprintf(stderr, "mt: Command not supported by this kernel.\n");
+	    else if (comp->error_tests != 0)
+		test_error(mtfd, comp);
+	}
+    }
     else {
 	fprintf(stderr, "mt: Internal error: command without function.\n");
 	i = 1;
@@ -302,7 +381,7 @@ usage(int explain)
     int ind;
     char line[100];
 
-    fprintf(stderr, "usage: mt [-v] [-h] [ -f device ] command [ count ]\n");
+    fprintf(stderr, "usage: mt [-v] [--version] [-h] [ -f device ] command [ count ]\n");
     if (explain) {
 	for (ind=0; cmds[ind].cmd_name != NULL; ) {
 	    if (ind == 0)
@@ -510,11 +589,15 @@ do_status(int mtfd, cmdef_tr *cmd, int argc, char **argv)
 	type = "SCSI 1";
     else if (status.mt_type == MT_ISSCSI2)
 	type = "SCSI 2";
+    else if (status.mt_type == MT_ISONSTREAM_SC)
+	type = "OnStream SC-, DI-, DP-, or USB";
     else
 	type = NULL;
     if (type == NULL) {
 	if (status.mt_type & 0x800000)
 	    printf ("qic-117 drive type = 0x%05lx\n", status.mt_type & 0x1ffff);
+	else if (status.mt_type == 0)
+	    printf("IDE-Tape (type code 0) ?\n");
 	else
 	    printf("Unknown tape drive type (type code %ld)\n", status.mt_type);
 	printf("File number=%d, block number=%d.\n",
@@ -533,10 +616,11 @@ do_status(int mtfd, cmdef_tr *cmd, int argc, char **argv)
 	    printf("File number=%d, block number=%d.\n",
 		   status.mt_fileno, status.mt_blkno);
 	if (status.mt_type == MT_ISSCSI1 ||
-	    status.mt_type == MT_ISSCSI2) {
+	    status.mt_type == MT_ISSCSI2 ||
+	    status.mt_type == MT_ISONSTREAM_SC) {
 	    dens = (status.mt_dsreg & MT_ST_DENSITY_MASK) >> MT_ST_DENSITY_SHIFT;
-	    density = "unknown to this mt";
-	    for (i=0; density_tbl[i].code >= 0; i++)
+	    density = "no translation";
+	    for (i=0; i < NBR_DENSITIES; i++)
 		if (density_tbl[i].code == dens) {
 		    density = density_tbl[i].name;
 		    break;
@@ -584,180 +668,36 @@ do_status(int mtfd, cmdef_tr *cmd, int argc, char **argv)
 	static int
 print_densities(int fd, cmdef_tr *cmd, int argc, char **argv)
 {
-    int i;
+    int i, offset;
 
-    printf("Some SCSI tape density codes:\ncode   explanation\n");
-    for (i=0; density_tbl[i].code >= 0; i++)
-	printf("0x%02x   %s\n", density_tbl[i].code, density_tbl[i].name);
+    printf("Some SCSI tape density codes:\ncode   explanation                   code   explanation\n");
+    offset = (NBR_DENSITIES + 1) / 2;
+    for (i=0; i < offset; i++) {
+	printf("0x%02x   %-28s", density_tbl[i].code, density_tbl[i].name);
+	if (i + offset < NBR_DENSITIES)
+	    printf("  0x%02x   %s\n", density_tbl[i + offset].code, density_tbl[i + offset].name);
+	else
+	    printf("\n");
+    }
     return 0;
 }
 
 
-
-#if DATCOMPRESSION
-
-/*** Get and set the DAT compression (Mode Page 15) ***/
-
-#if 0
-#include <linux/scsi.h>
-#else
-/* The file scsi.h is not visible to the users after 1.3.98. The necessary
-   SCSI constants are defined here (not so bad because these are constants
-   defined in the SCSI standard. */
-#define MODE_SELECT           0x15
-#define MODE_SENSE            0x1a
-#endif
-
-#ifndef SCSI_IOCTL_SEND_COMMAND
-/* This has not been defined in a public include file */
-#define SCSI_IOCTL_SEND_COMMAND 1
-#endif
-
-#define COMPRESSION_PAGE        0x0f
-#define COMPRESSION_PAGE_LENGTH 16
-
-#define IOCTL_HEADER_LENGTH 8
-#define CMD_6_LENGTH        6
-#define MODE_HEADER_OFFSET  (IOCTL_HEADER_LENGTH + CMD_6_LENGTH)
-#define MODE_HEADER_LENGTH  4
-
-#define DCE_MASK  0x80
-#define DCC_MASK  0x40
-#define RED_MASK  0x60
-
-
-static int read_mode_page(int fn, int page, int length, unsigned char *buffer,
-			  int do_mask)
+/* Try to find out why the command failed */
+	static void
+test_error(int mtfd, cmdef_tr *cmd)
 {
-    int result, *ip;
-    unsigned char tmpbuffer[30], *cmd;
+    struct mtget status;
 
-    memset(tmpbuffer, 0, IOCTL_HEADER_LENGTH + CMD_6_LENGTH);
-    ip = (int *)&(tmpbuffer[0]);
-    *ip = 0;
-    *(ip+1) = length + MODE_HEADER_LENGTH;
+    if (ioctl(mtfd, MTIOCGET, (char *)&status) < 0)
+	return;
 
-    cmd = &(tmpbuffer[8]);
-    cmd[0] = MODE_SENSE;
-    cmd[1] = 8;
-    cmd[2] = page;
-    if (do_mask)
-	cmd[2] |= 0x40;  /* Get changeable parameter mask */
-    cmd[4] = length + MODE_HEADER_LENGTH;
+    if (status.mt_type != MT_ISSCSI1 && status.mt_type != MT_ISSCSI2)
+	return;
 
-    result = ioctl(fn, SCSI_IOCTL_SEND_COMMAND, tmpbuffer);
-    if (result) {
-	if (getuid() != 0)
-	    fprintf(stderr, "%s: This operation is only allowed for root.\n",
-		    tape_name);
-	else
-	    fprintf(stderr, "%s: Can't read SCSI mode page.\n", tape_name);
-	return 0;
-    }
-    memcpy(buffer, tmpbuffer + IOCTL_HEADER_LENGTH, length + MODE_HEADER_LENGTH);
-    return 1;
+    if ((cmd->error_tests & ET_ONLINE) && !GMT_ONLINE(status.mt_gstat))
+	fprintf(stderr, "mt: The device is offline (not powered on, no tape ?).\n");
+    if ((cmd->error_tests & ET_WPROT) && !GMT_WR_PROT(status.mt_gstat))
+	fprintf(stderr, "mt: The tape is write-protected.\n");
 }
 
-
-static int write_mode_page(int fn, int page, int length, unsigned char *buffer)
-{
-    int result, *ip;
-    unsigned char tmpbuffer[40], *cmd;
-
-    memset(tmpbuffer, 0, IOCTL_HEADER_LENGTH + CMD_6_LENGTH);
-    ip = (int *)&(tmpbuffer[0]);
-    *ip = length + MODE_HEADER_LENGTH;
-    *(ip+1) = 0;
-
-    cmd = &(tmpbuffer[IOCTL_HEADER_LENGTH]);
-    cmd[0] = MODE_SELECT;
-    cmd[1] = 0x10;
-    cmd[4] = length + MODE_HEADER_LENGTH;
-
-    memcpy(tmpbuffer + MODE_HEADER_OFFSET, buffer, length + MODE_HEADER_LENGTH);
-    tmpbuffer[MODE_HEADER_OFFSET] = 0;      /* reserved data length */
-    tmpbuffer[MODE_HEADER_OFFSET + 1] = 0;  /* reserved meadia type */
-    tmpbuffer[MODE_HEADER_OFFSET + MODE_HEADER_LENGTH] &= 0x3f; 
-                                 /* reserved bits in page code byte */
-
-    result = ioctl(fn, SCSI_IOCTL_SEND_COMMAND, tmpbuffer);
-    if (result) {
-	fprintf(stderr, "%s: Can't write mode page.\n", tape_name);
-	return 0;
-    }
-    return 1;
-}
-
-
-	static int
-do_dat_compression(int fn, cmdef_tr *cmd, int argc, char **new)
-{
-    int i, alter = (argc > 0);
-    unsigned char buffer[30], mask[30];
-    struct mtpos mt_pos;
-
-    if (!read_mode_page(fn, COMPRESSION_PAGE, COMPRESSION_PAGE_LENGTH,
-			buffer, 0)) {
-	fprintf(stderr, "%s: Can't read compression mode page.\n", tape_name);
-	return 2;
-    }
-
-    if (alter) {
-	if (!read_mode_page(fn, COMPRESSION_PAGE, COMPRESSION_PAGE_LENGTH,
-			    mask, 1)) {
-	    fprintf(stderr, "%s: Can't read changeable parameter page.\n", tape_name);
-	    return 2;
-	}
-
-	if ((buffer[MODE_HEADER_LENGTH + 2] & DCC_MASK) == 0) {
-	    fprintf(stderr, "%s: Drive does not support compression.\n", tape_name);
-	    return 2;
-	}
-	if ((mask[MODE_HEADER_LENGTH + 2] & DCE_MASK) == 0) {
-	    fprintf(stderr, "%s: Compression not controllable with software.\n",
-		    tape_name);
-	    return 2;
-	}
-
-	if (((buffer[MODE_HEADER_LENGTH + 3] & RED_MASK) >> 5) == 2) {
-	    if (ioctl(fn, MTIOCPOS, (char *)&mt_pos) >= 0
-		&& mt_pos.mt_blkno != 0) {
-		fprintf(stderr, "%s: Can't change compression here. Please rewind.\n",
-			tape_name);
-		return 2;
-	    }
-	}
-
-	if (**new == '0' || !strncmp(*new, "off", 3))
-	    buffer[MODE_HEADER_LENGTH + 2] &= ~DCE_MASK;
-	else
-	    buffer[MODE_HEADER_LENGTH + 2] |= DCE_MASK;
-
-	if (!write_mode_page(fn, COMPRESSION_PAGE, COMPRESSION_PAGE_LENGTH,
-			     buffer)) {
-	    for (i=2; i < 16; i++)
-		buffer[MODE_HEADER_LENGTH + i] &= mask[MODE_HEADER_LENGTH + i];
-	    if (!write_mode_page(fn, COMPRESSION_PAGE, COMPRESSION_PAGE_LENGTH,
-				 buffer)) {
-		fprintf(stderr, "%s: Writing mode SCSI mode page failed.\n", tape_name);
-		fprintf(stderr, "%s: Compression mode not changed.\n", tape_name);
-		return 2;
-	    }
-	}
-	if (!read_mode_page(fn, COMPRESSION_PAGE, COMPRESSION_PAGE_LENGTH,
-			    buffer, 0)) {  /* Re-read to check */
-	    fprintf(stderr, "%s: Re-read of the compression page failed.\n", tape_name);
-	    return 2;
-	}
-    }
-
-    if (buffer[4+2] & 0x80)
-	printf("Compression on.\n");
-    else
-	printf("Compression off.\n");
-
-    close(fn);
-    return 0;
-}
-
-#endif
