@@ -1,26 +1,31 @@
 /* This program initializes Linux SCSI tape drives using the
    inquiry data from the devices and a text database.
 
-   Copyright 1996-1998 by Kai Mkisara (email Kai.Makisara@metla.fi)
+   Copyright 1996-2000 by Kai Mkisara (email Kai.Makisara@metla.fi)
    Distribution of this program is allowed according to the
    GNU Public Licence.
 
-   Last modified: Sat Aug 15 12:18:13 1998 by makisara@home
+   Last modified: Mon Nov 27 21:17:39 2000 by makisara@kai.makisara.local
 */
 
+#ifndef lint
+static char rcsid[] = "$Id: /home/makisara/src/sys/mt-st-0.6/stinit.c at Mon Nov 27 21:17:39 2000 by makisara@kai.makisara.local$";
+#endif
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 #include <ctype.h>
 #include <unistd.h>
+#include <errno.h>
 #include <fcntl.h>
 #include <dirent.h>
-#include <errno.h>
 #include <sys/stat.h>
-#include <sys/mtio.h>
+#include <sys/ioctl.h>
 #include <sys/sysmacros.h>
 #include <linux/major.h>
-#include <scsi/scsi.h>
+
+#include "mtio.h"
 
 #ifndef FALSE
 #define TRUE 1
@@ -28,6 +33,7 @@
 #endif
 #define SKIP_WHITE(p) for ( ; *p == ' ' || *p == '\t'; p++)
 
+#define VERSION "0.6"
 
 typedef struct _modepar_tr {
     int defined;
@@ -203,7 +209,7 @@ next_block(FILE *dbf, char *buf, int buflen, int limiter)
 	SKIP_WHITE(cp);
 	strcpy(bp, cp);
 	strcat(bp, " ");
-	len += strlen(lbuf) + 1;
+	len += strlen(cp) + 1;
 	if (fgets(lbuf, LINEMAX, dbf) == NULL)
 	    return FALSE;
 	if ((cp = strchr(lbuf, '#')) != NULL)
@@ -408,6 +414,8 @@ find_pars(FILE *dbf, char *company, char *product, char *rev, devdef_tr *defs,
 }
 
 
+#define INQUIRY 0x12
+
 #ifndef SCSI_IOCTL_SEND_COMMAND
 #define SCSI_IOCTL_SEND_COMMAND 1
 #endif
@@ -421,7 +429,7 @@ do_inquiry(char *tname, char *company, char *product, char *rev, int print_non_f
 #define BUFLEN 256
     unsigned char buffer[BUFLEN], *cmd;
 
-    if ((fn = open(tname, O_RDONLY)) < 0) {
+    if ((fn = open(tname, O_RDONLY | O_NONBLOCK)) < 0) {
 	if (print_non_found || verbose > 0) {
 	    if (errno == ENXIO)
 		fprintf(stderr, "Device '%s' not found by kernel.\n", tname);
@@ -473,7 +481,7 @@ do_inquiry(char *tname, char *company, char *product, char *rev, int print_non_f
 	static int
 tapenum(char *name)
 {
-    int dev, found;
+    int dev;
     struct dirent *dent;
     DIR *dirp;
     char *dn, **dnp, tmpname[PATH_MAX];
@@ -491,7 +499,7 @@ tapenum(char *name)
 	return dev;
     }
     else { /* Search from the device directories */
-	for (dnp=devdirs, found=0; found < NBR_MODES && *dnp != NULL; dnp++) {
+	for (dnp=devdirs; *dnp != NULL; dnp++) {
 	    dn = *dnp;
 	    if ((dirp = opendir(dn)) == NULL)
 	      continue;
@@ -517,8 +525,6 @@ tapenum(char *name)
 	}
     }
 
-    if (found > 0)
-	return dev;
     return (-1);
 }
 
@@ -582,7 +588,8 @@ set_defs(devdef_tr *defs, char **fnames)
     for (i=0; i < NBR_MODES; i++) {
 	if (*fnames[i] == '\0' || !defs->modedefs[i].defined)
 	    continue;
-	if ((tape = open(fnames[i], O_RDONLY)) < 0) {
+
+	if ((tape = open(fnames[i], O_RDONLY | O_NONBLOCK)) < 0) {
 	    fprintf(stderr, "Can't open the tape device '%s' for mode %d.\n",
 		    fnames[i], i);
 	    return FALSE;
@@ -783,6 +790,12 @@ main(int argc, char **argv)
 		usage(1);
 	    dbname = argv[argn];
 	}
+	else if (*(argv[argn] + 1) == '-' &&
+		 *(argv[argn] + 2) == 'v') {
+	    printf("stinit v. %s\n", VERSION);
+	    exit(0);
+	    break;
+	}
 	else
 	    usage(1);
     }
@@ -800,8 +813,10 @@ main(int argc, char **argv)
 
     if (argc > argn) {  /* Initialize specific drives */
 	for ( ; argn < argc; argn++) {
-	    if (*argv[argn] == '-')
+	    if (*argv[argn] == '-') {
 		usage(1);
+		return 1; /* Never executed but makes gcc happy */
+	    }
 	    else if (isdigit(*argv[argn]))
 		tapeno = strtol(argv[argn], NULL, 0);
 	    else if ((tapeno = tapenum(argv[argn])) < 0) {
